from click.testing import CliRunner
from pipeglow.cli import cli


def test_version():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ["--version"])
        assert result.exit_code == 0
        assert result.output.startswith("cli, version ")



import os
import uuid
from .utils import send_govee_request

def update_govee_light_brightness(sku, device, brightness):
    """
    ✨ Update the brightness of a particular Govee light via the REST

    :param: Govee SKU for the light to update
    :param: Govee Device ID (unique identifier) for the light to update
    :param: The brightness to set
    """
    url = "https://openapi.api.govee.com/router/api/v1/device/control"
    headers = {
        "Govee-API-Key": os.getenv("GOVEE_API_KEY"),
        "Content-Type": "application/json"
    }
    payload = {
        "requestId": str(uuid.uuid4()),
        "payload": {
            "sku": sku,
            "device": device,
            "capability": {
                "type": "devices.capabilities.brightness",
                "instance": "brightness",
                "value": brightness,
                }
            }
        }
    send_govee_request(url, headers, payload)

def update_govee_light_color(sku, device, color_rgb):
    """
    ✨ Update the color of a particular Govee light via the REST API

    :param: Govee SKU for the light to update
    :param: Govee Device ID (unique identifier) for the light to update
    :param: The color to set
    """
    url = "https://openapi.api.govee.com/router/api/v1/device/control"
    headers = {
        "Govee-API-Key": os.getenv("GOVEE_API_KEY"),
        "Content-Type": "application/json"
    }
    payload = {
        "requestId": str(uuid.uuid4()),
        "payload": {
            "sku": sku,
            "device": device,
            "capability": {
                "type": "devices.capabilities.color_setting",
                "instance": "colorRgb",
                "value": color_rgb,
                }
            }
        }
    send_govee_request(url, headers, payload)


import click
import emoji
from dotenv import load_dotenv
from .gitlab_api import fetch_pipeline_status_and_time
from .govee_api import update_govee_light_color, update_govee_light_brightness
from .utils import determine_light_color, calculate_temperature_and_brightness, get_govee_device_info
@click.group()
@click.version_option()
def cli():
    """
    ✨ PIPEGLOW | Change smart lights based on CI pipeline status
    """

@cli.command(name="change_the_lights")
@click.option('--gitlab-url', default='https://gitlab.com', show_default=True, help='URL of the GitLab instance')
def change_the_lights(gitlab_url):
    """
    ✨ The main loop for PIPEGLOW ✨

    Retrieve status and "updated_at" for the latest pipeline in a particular project
    Calculate a color and brightness based on the status and "updated_at"
    Update the smart light based on this information

    :param: URL for the GitLab instance that will be queried, defaults to https://gitlab.com.
    """
    # 🔬 Gather variables for this environment
    load_dotenv()
    # 🔭 Get the status and "updated_at" for the most recently created pipeline
    pipeline_status, pipeline_updated_at = fetch_pipeline_status_and_time(gitlab_url)
    # 🎨 Determine what color to set the light to based on the pipeline status
    color_rgb = determine_light_color(pipeline_status)
    # ☀️ Determine how bright the light should be based on how recently the pipeline finished
    ## 🔎 More recent = brighter light
    light_brightness = calculate_temperature_and_brightness(pipeline_updated_at)
    # 💡 Collect the Govee SKU and Device information
    govee_sku, govee_device = get_govee_device_info()
    click.echo(emoji.emojize(f":rocket: The latest pipeline was {pipeline_status} so we will set the light to {color_rgb}!"))
    # 🌈 Set the color
    update_govee_light_color(govee_sku, govee_device, color_rgb)
    # 🚀 Set the brightness
    update_govee_light_brightness(govee_sku, govee_device, light_brightness)

@cli.command(name="set_the_lights")
@click.option('--light-status', default="success", show_default=True, help="Set the light status directly, useful if you are not using pipeglow to check CI.")
def set_the_lights(light_status):
    """
    🎯 Set light color based on a pipeline status 🎯
    """
    click.echo(emoji.emojize(f":rocket: Just set the lights to {light_status} please!"))
    # 🔬 Gather variables for this environment
    load_dotenv()
    # 💡 Collect the Govee SKU and Device information
    govee_sku, govee_device = get_govee_device_info()
    # 🧮 Calculate the color based on the light status that was provided
    color_rgb = determine_light_color(light_status)
    # 🌈 Set the color
    update_govee_light_color(govee_sku, govee_device, color_rgb)


from datetime import datetime, timezone, timedelta
import requests
import json
import click
import emoji
import os

def determine_light_color(status):
    """
    Determines the color the light should be based on the pipeline status.

    :param status: The pipeline status (e.g., 'success', 'failed', 'running').
    :return: The RGB color value to use for the provided status.
    """
    color_map = {
        "created": "#9F9110",                  # getting ready yellow
        "waiting_for_resource": "#DC6BAD",     # light purplish pink
        "preparing": "#9F9110",                # getting ready yellow
        "pending":"#9F9110",                   # getting ready yellow
        "running": "#3974C6",                  # in progress blue
        "success": "#309508",                  # success green
        "failed": "#FF0000",                   # error red
        "canceled": "#212121",                 # never mind gray
        "skipped": "#212121",                   # never mind gray
    }
    color = color_map.get(status, "#FFFFFF")
    color_rgb = int(color.lstrip('#'), 16)
    return color_rgb  # Default to white if status is unknown

def calculate_temperature_and_brightness(updated_at):
    """
    Calculates the temperature and brightness the light should be based on how recently the pipeline was updated.
    The more recently the pipeline was updated, the brighter the light will be. This makes more of a difference
    for side projects that might have a lot of pipelines.

    :param updated_at: The datetime when the pipeline was last updated (string format).
    :return: A tuple (temperature, brightness).
    """
    # time and date calculations
    updated_at = datetime.fromisoformat(updated_at)
    now = datetime.utcnow().replace(tzinfo=timezone.utc)
    delta = now - updated_at

    # The Govee API docs tell us that the temperature can range from 2000 to 9000
    if delta < timedelta(minutes=5):
        temperature = 9000
        brightness = 100
    elif delta < timedelta(minutes=15):
        temperature = 7000
        brightness = 90
    elif delta < timedelta(hours=1):
        temperature = 5000
        brightness = 75
    elif delta < timedelta(days=1):
        temperature = 4000
        brightness = 50
    elif delta < timedelta(weeks=1):
        temperature = 3000
        brightness = 30
    else:
        temperature = 2000
        brightness = 10
    return temperature, brightness

# Centralized function for sending API requests and handling errors
def send_govee_request(url, headers, payload):
    """
    Send a request to the Govee API and handle errors.

    :param url: The Govee API endpoint to use for this call. (All values today are the same.)
    :param headers: The HTTP headers to pass along with this request, this is how authentication to the Govee API is handled.
    :param payload:  The payload for the POST request to the Govee API.
    """
    try:
        response = requests.post(url, headers=headers, data=json.dumps(payload))
        if response.status_code == 200:
            click.echo(emoji.emojize(":trophy: The light has been updated!"))
        elif response.status_code == 400:
            click.echo(emoji.emojize(
                ":warning: Error: A required parameter is missing. Check the payload and try again."))
        elif response.status_code == 404:
            click.echo(emoji.emojize(":x: Error: Device not found. Check the device ID and try again."))
        elif response.status_code == 429:
            click.echo(emoji.emojize(
                ":stopwatch: You hit the Govee API request limit of 10000 requests per account per day. Try again later."))
        else:
            click.echo(emoji.emojize(f":warning: Unexpected error: {response.status_code} - {response.text}"))
    except requests.exceptions.SSLError as e:
        click.echo(emoji.emojize(f":fox: SSL Error: {str(e)} - Check your SSL certificate or try again later."))
    except requests.exceptions.RequestException as e:
        click.echo(emoji.emojize(f":warning: : {str(e)}"))

def get_govee_device_info():
    """
    Fetches the device information (Govee SKU, Device ID) and sets up headers.

    :return: A tuple (Govee SKU, device).
    """
    # Get the device information from environment variables
    sku = os.getenv("GOVEE_DEVICE_MODEL")
    device = os.getenv("GOVEE_DEVICE_ID")
    if not sku or not device:
        raise ValueError(emoji.emojize(":crying_cat: The Govee Device SKU or ID is missing from the environment variables."))
    return sku, device


import click
import emoji
import gitlab
import os

def fetch_pipeline_status_and_time(gitlab_url):
    """
    ✨ Fetch the latest pipeline status and its updated_at timestamp.

    :param: URL for the GitLab instance that will be queried
    :return: A tuple of information about the pipeline that was identified (status, updated_at).
    """
    # Get GitLab token and project ID from environment variables
    gitlab_token = os.getenv('GITLAB_API_TOKEN')
    project_id = os.getenv('GITLAB_PROJECT_ID')
    if not gitlab_token:
        raise click.ClickException(emoji.emojize(":cross_mark_button: GITLAB_API_TOKEN environment variable not set"))
    if not project_id:
        raise click.ClickException(emoji.emojize(":cross_mark_button: GITLAB_PROJECT_ID environment variable not set"))
    gtlb = gitlab.Gitlab(gitlab_url, private_token=gitlab_token)
    try:
        project = gtlb.projects.get(project_id)
        # ℹ️ Get the latest pipeline in the list of pipelines | Explore alternate paths later
        pipelines = project.pipelines.list(per_page=1, get_all=False)
        if not pipelines:
            raise click.ClickException(emoji.emojize(":cross_mark_button: No pipelines found for the project"))
        latest_pipeline = pipelines[0]
        status = latest_pipeline.status
        updated_at = latest_pipeline.updated_at
        return status, updated_at
    except gitlab.exceptions.GitlabError as e:
        raise click.ClickException(emoji.emojize(f":fox: GitLab API error encountered: {str(e)}"))